/*
    tcl-fuse - Tcl FUSE Interface
    A Tcl interface to the Linux kernel's FUSE subsystem.

    Copyright (C) 2008  Alexandros Stergiakis

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * proxy.h
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <fuse.h>

void _destroy(void *user_data);
int _getattr(const char *path, struct stat *st);
int _readdir(const char *path, void *buf, fuse_fill_dir_t filler, off_t offset, struct fuse_file_info *fi);
int _open(const char *path, struct fuse_file_info *fi);
int _read(const char *path, char *buf, size_t size, off_t offset, struct fuse_file_info *fi);
int _readlink(const char *path, char *buf, size_t size);
int _rename(const char *from, const char *to);
int _mknod(const char *path, mode_t mode, dev_t rdev);
int _mkdir(const char *path, mode_t mode);
int _unlink(const char *path);
int _rmdir(const char *path);
int _symlink(const char *from, const char *to);
int _link(const char *from, const char *to);
int _chmod(const char *path, mode_t mode);
int _chown(const char *path, uid_t uid, gid_t gid);
int _truncate(const char *path, off_t size);
int _write(const char *path, const char *buf, size_t size, off_t offset, struct fuse_file_info *fi);
int _statfs(const char *path, struct statvfs *stbuf);
int _release(const char *path, struct fuse_file_info *fi);
int _fsync(const char *path, int isdatasync, struct fuse_file_info *fi);
int _setxattr(const char *path, const char *name, const char *value, size_t size, int flags);
int _getxattr(const char *path, const char *name, char *value, size_t size);
int _listxattr(const char *path, char *list, size_t size);
int _removexattr(const char *path, const char *name);
int _access(const char *path, int mask);
int _flush(const char *path, struct fuse_file_info *fi);
int _opendir(const char *path, struct fuse_file_info *fi);
int _releasedir(const char *path, struct fuse_file_info *fi);
int _fsyncdir(const char *path, int datasync, struct fuse_file_info *fi);
int _create(const char *path, mode_t mode, struct fuse_file_info *fi);
int _ftruncate(const char *path, off_t size, struct fuse_file_info *fi);
int _fgetattr(const char *path, struct stat *st, struct fuse_file_info *fi);
int _lock(const char *path, struct fuse_file_info *fi, int cmd, struct flock *lock);
int _utimens(const char *path, const struct timespec tv[]);
int _bmap(const char *path, size_t blocksize, uint64_t *idx);
